@@ -4,6 +4,12 @@ import (
 	"fmt"
 	"time"
 
+	v1 "k8s.io/api/core/v1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/apimachinery/pkg/runtime/serializer"
+	"k8s.io/apimachinery/pkg/util/wait"
+
 	g "github.com/onsi/ginkgo"
 	o "github.com/onsi/gomega"
 
@@ -11,11 +17,6 @@ import (
 	configv1 "github.com/openshift/api/config/v1"
 	operatorv1 "github.com/openshift/api/operator/v1"
 	exutil "github.com/openshift/origin/test/extended/util"
-	v1 "k8s.io/api/core/v1"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/runtime"
-	"k8s.io/apimachinery/pkg/runtime/serializer"
-	"k8s.io/apimachinery/pkg/util/wait"
 )
 
 // e2e tests of the build controller configuration.
@@ -55,7 +56,7 @@ var _ = g.Describe("[Feature:Builds][Serial][Slow][Disruptive] alter builds via
 		checkOCMProgressing = func(progressing operatorv1.ConditionStatus) {
 			g.By("check that the OCM enters Progressing==" + string(progressing))
 			var err error
-			err = wait.Poll(1*time.Second, 2*time.Minute, func() (bool, error) {
+			err = wait.Poll(5*time.Second, 10*time.Minute, func() (bool, error) {
 				ocm, err := oc.AdminOperatorClient().OperatorV1().OpenShiftControllerManagers().Get("cluster", metav1.GetOptions{})
 				if err != nil {
 					g.By("intermediate error accessing ocm: " + err.Error())
@@ -70,34 +71,6 @@ var _ = g.Describe("[Feature:Builds][Serial][Slow][Disruptive] alter builds via
 			})
 			o.Expect(err).NotTo(o.HaveOccurred())
 		}
-		checkDSRolloutState = func(inProgress bool) {
-			g.By(fmt.Sprintf("check that a OCM DS rollout being in progress is %v", inProgress))
-			var err error
-			err = wait.Poll(1*time.Second, 2*time.Minute, func() (bool, error) {
-				ds, err := oc.AdminKubeClient().AppsV1().DaemonSets("openshift-controller-manager").Get("controller-manager", metav1.GetOptions{})
-				if err != nil {
-					g.By("intermediate error access ds: " + err.Error())
-					return false, nil
-				}
-				if inProgress {
-					if ds.Status.CurrentNumberScheduled < ds.Status.DesiredNumberScheduled ||
-						ds.Status.NumberReady < ds.Status.DesiredNumberScheduled ||
-						ds.Status.NumberAvailable < ds.Status.DesiredNumberScheduled ||
-						ds.Status.UpdatedNumberScheduled < ds.Status.DesiredNumberScheduled {
-						return true, nil
-					}
-					return false, nil
-				}
-				if ds.Status.CurrentNumberScheduled == ds.Status.DesiredNumberScheduled &&
-					ds.Status.NumberReady == ds.Status.DesiredNumberScheduled &&
-					ds.Status.NumberAvailable == ds.Status.DesiredNumberScheduled &&
-					ds.Status.UpdatedNumberScheduled == ds.Status.DesiredNumberScheduled {
-					return true, nil
-				}
-				return false, nil
-			})
-			o.Expect(err).NotTo(o.HaveOccurred())
-		}
 		checkBuildPodUnschedulable = func(name string) {
 			g.By(fmt.Sprintf("check the build pod %s is unschedulable", name))
 			var err error
@@ -160,7 +133,7 @@ var _ = g.Describe("[Feature:Builds][Serial][Slow][Disruptive] alter builds via
 			oc.Run("create").Args("-f", buildFixture2).Execute()
 		})
 
-		g.AfterEach(func() {
+		g.JustAfterEach(func() {
 			if g.CurrentGinkgoTestDescription().Failed {
 				exutil.DumpPodStates(oc)
 				exutil.DumpPodLogsStartingWith("", oc)
@@ -170,6 +143,7 @@ var _ = g.Describe("[Feature:Builds][Serial][Slow][Disruptive] alter builds via
 
 		g.Context("registries config context", func() {
 
+			// Altering registries config does not force an OCM rollout
 			g.AfterEach(func() {
 				oc.AsAdmin().Run("apply").Args("-f", defaultConfigFixture).Execute()
 			})
@@ -228,8 +202,7 @@ var _ = g.Describe("[Feature:Builds][Serial][Slow][Disruptive] alter builds via
 
 		})
 
-		g.Context("build config context", func() {
-
+		g.Context("build config no ocm rollout", func() {
 			g.AfterEach(func() {
 				g.By("reset build cluster configuration")
 				buildConfig, err := oc.AdminConfigClient().ConfigV1().Builds().Get("cluster", metav1.GetOptions{})
@@ -328,6 +301,21 @@ var _ = g.Describe("[Feature:Builds][Serial][Slow][Disruptive] alter builds via
 				o.Expect(*build.Spec.Source.Git.HTTPSProxy).To(o.Equal(buildConfig.Spec.BuildDefaults.GitProxy.HTTPSProxy))
 				o.Expect(*build.Spec.Source.Git.NoProxy).To(o.Equal(buildConfig.Spec.BuildDefaults.GitProxy.NoProxy))
 			})
+		})
+
+		g.Context("build config with ocm rollout", func() {
+
+			g.AfterEach(func() {
+				g.By("reset build cluster configuration")
+				buildConfig, err := oc.AdminConfigClient().ConfigV1().Builds().Get("cluster", metav1.GetOptions{})
+				o.Expect(err).NotTo(o.HaveOccurred())
+				buildConfig.Spec.BuildDefaults = configv1.BuildDefaults{}
+				buildConfig.Spec.BuildOverrides = configv1.BuildOverrides{}
+				_, err = oc.AdminConfigClient().ConfigV1().Builds().Update(buildConfig)
+				o.Expect(err).NotTo(o.HaveOccurred())
+				checkOCMProgressing(operatorv1.ConditionTrue)
+				checkOCMProgressing(operatorv1.ConditionFalse)
+			})
 
 			// this replaces coverage from the TestBuildDefaultEnvironment integration test
 			g.It("Apply env configuration to build pod", func() {
@@ -348,11 +336,6 @@ var _ = g.Describe("[Feature:Builds][Serial][Slow][Disruptive] alter builds via
 				o.Expect(err).NotTo(o.HaveOccurred())
 				checkOCMProgressing(operatorv1.ConditionTrue)
 				checkOCMProgressing(operatorv1.ConditionFalse)
-				checkDSRolloutState(true)
-				checkDSRolloutState(false)
-				// there is no way programmatically to check leader election outcome
-				g.By("waiting 10s for controller-manager leader election to complete")
-				time.Sleep(10 * time.Second)
 				g.By("verify build.config is set")
 				buildConfig, err = oc.AdminConfigClient().ConfigV1().Builds().Get("cluster", metav1.GetOptions{})
 				o.Expect(err).NotTo(o.HaveOccurred())
@@ -406,11 +389,6 @@ var _ = g.Describe("[Feature:Builds][Serial][Slow][Disruptive] alter builds via
 				o.Expect(err).NotTo(o.HaveOccurred())
 				checkOCMProgressing(operatorv1.ConditionTrue)
 				checkOCMProgressing(operatorv1.ConditionFalse)
-				checkDSRolloutState(true)
-				checkDSRolloutState(false)
-				// there is no way programmatically to check leader election outcome
-				g.By("waiting 10s for controller-manager leader election to complete")
-				time.Sleep(10 * time.Second)
 				g.By("verify build.config is set")
 				buildConfig, err = oc.AdminConfigClient().ConfigV1().Builds().Get("cluster", metav1.GetOptions{})
 				o.Expect(err).NotTo(o.HaveOccurred())
@@ -465,11 +443,6 @@ var _ = g.Describe("[Feature:Builds][Serial][Slow][Disruptive] alter builds via
 				o.Expect(err).NotTo(o.HaveOccurred())
 				checkOCMProgressing(operatorv1.ConditionTrue)
 				checkOCMProgressing(operatorv1.ConditionFalse)
-				checkDSRolloutState(true)
-				checkDSRolloutState(false)
-				// there is no way programmatically to check leader election outcome
-				g.By("waiting 10s for controller-manager leader election to complete")
-				time.Sleep(10 * time.Second)
 				g.By("verify build.config is set")
 				buildConfig, err = oc.AdminConfigClient().ConfigV1().Builds().Get("cluster", metav1.GetOptions{})
 				o.Expect(err).NotTo(o.HaveOccurred())
@@ -518,11 +491,6 @@ var _ = g.Describe("[Feature:Builds][Serial][Slow][Disruptive] alter builds via
 
 				checkOCMProgressing(operatorv1.ConditionTrue)
 				checkOCMProgressing(operatorv1.ConditionFalse)
-				checkDSRolloutState(true)
-				checkDSRolloutState(false)
-				// there is no way programmatically to check leader election outcome
-				g.By("waiting 10s for controller-manager leader election to complete")
-				time.Sleep(10 * time.Second)
 				g.By("verify build.config is set")
 				buildConfig, err = oc.AdminConfigClient().ConfigV1().Builds().Get("cluster", metav1.GetOptions{})
 				o.Expect(err).NotTo(o.HaveOccurred())
@@ -571,11 +539,6 @@ var _ = g.Describe("[Feature:Builds][Serial][Slow][Disruptive] alter builds via
 
 				checkOCMProgressing(operatorv1.ConditionTrue)
 				checkOCMProgressing(operatorv1.ConditionFalse)
-				checkDSRolloutState(true)
-				checkDSRolloutState(false)
-				// there is no way programmatically to check leader election outcome
-				g.By("waiting 10s for controller-manager leader election to complete")
-				time.Sleep(10 * time.Second)
 				g.By("verify build.config is set")
 				buildConfig, err = oc.AdminConfigClient().ConfigV1().Builds().Get("cluster", metav1.GetOptions{})
 				o.Expect(err).NotTo(o.HaveOccurred())
