@@ -49,9 +49,11 @@ import (
 	configv1 "github.com/openshift/api/config/v1"
 	imagev1 "github.com/openshift/api/image/v1"
 	operatorv1 "github.com/openshift/api/operator/v1"
+	securityv1 "github.com/openshift/api/security/v1"
 	appsv1clienttyped "github.com/openshift/client-go/apps/clientset/versioned/typed/apps/v1"
 	buildv1clienttyped "github.com/openshift/client-go/build/clientset/versioned/typed/build/v1"
 	imagev1typedclient "github.com/openshift/client-go/image/clientset/versioned/typed/image/v1"
+	projectv1typedclient "github.com/openshift/client-go/project/clientset/versioned/typed/project/v1"
 	"github.com/openshift/library-go/pkg/apps/appsutil"
 	"github.com/openshift/library-go/pkg/build/naming"
 	"github.com/openshift/library-go/pkg/git"
@@ -249,11 +251,6 @@ func WaitForOpenShiftNamespaceImageStreams(oc *CLI) error {
 				e2e.Logf("SamplesOperator degraded!!!")
 				return false
 			case condition.Type == configv1.OperatorProgressing:
-				if condition.Status == configv1.ConditionTrue {
-					// updates still in progress ... not "ready"
-					e2e.Logf("SamplesOperator still in progress")
-					return false
-				}
 				// if the imagestreams for one of our langs above failed, we abort,
 				// but if it is for say only EAP streams, we allow
 				if condition.Reason == "FailedImageImports" {
@@ -261,10 +258,41 @@ func WaitForOpenShiftNamespaceImageStreams(oc *CLI) error {
 					for _, lang := range langs {
 						if strings.Contains(msg, " "+lang+" ") || strings.HasSuffix(msg, " "+lang) {
 							e2e.Logf("SamplesOperator detected error during imagestream import: %s with details %s", condition.Reason, condition.Message)
+							stream, err := oc.AsAdmin().ImageClient().ImageV1().ImageStreams("openshift").Get(lang, metav1.GetOptions{})
+							if err != nil {
+								e2e.Logf("after seeing FailedImageImports for %s retrieval failed with %s", lang, err.Error())
+								return false
+							}
+							isi := &imagev1.ImageStreamImport{}
+							isi.Name = lang
+							isi.Namespace = "openshift"
+							isi.ResourceVersion = stream.ResourceVersion
+							isi.Spec = imagev1.ImageStreamImportSpec{
+								Import: true,
+								Images: []imagev1.ImageImportSpec{},
+							}
+							for _, tag := range stream.Spec.Tags {
+								if tag.From != nil && tag.From.Kind == "DockerImage" {
+									iis := imagev1.ImageImportSpec{}
+									iis.From = *tag.From
+									iis.To = &corev1.LocalObjectReference{Name: tag.Name}
+									isi.Spec.Images = append(isi.Spec.Images, iis)
+								}
+							}
+							_, err = oc.AsAdmin().ImageClient().ImageV1().ImageStreamImports("openshift").Create(isi)
+							if err != nil {
+								e2e.Logf("after seeing FailedImageImports for %s the manual image import failed with %s", lang, err.Error())
+							}
+							e2e.Logf("after seeing FailedImageImports for %s a manual image-import was submitted", lang)
 							return false
 						}
 					}
 				}
+				if condition.Status == configv1.ConditionTrue {
+					// updates still in progress ... not "ready"
+					e2e.Logf("SamplesOperator still in progress")
+					return false
+				}
 			case condition.Type == configv1.OperatorAvailable && condition.Status == configv1.ConditionFalse:
 				e2e.Logf("SamplesOperator not available")
 				return false
@@ -991,6 +1019,32 @@ func WaitForServiceAccount(c corev1client.ServiceAccountInterface, name string)
 	return wait.Poll(time.Duration(100*time.Millisecond), 3*time.Minute, waitFn)
 }
 
+// WaitForNamespaceSCCAnnotations waits up to 2 minutes for the cluster-policy-controller to add the SCC related
+// annotations to the provided namespace.
+func WaitForNamespaceSCCAnnotations(c projectv1typedclient.ProjectV1Interface, name string) error {
+	waitFn := func() (bool, error) {
+		proj, err := c.Projects().Get(name, metav1.GetOptions{})
+		if err != nil {
+			// it is assumed the project was created prior to calling this, so we
+			// do not distinguish not found errors
+			return false, err
+		}
+		if proj.Annotations == nil {
+			return false, nil
+		}
+		for k := range proj.Annotations {
+			// annotations to check based off of
+			// https://github.com/openshift/cluster-policy-controller/blob/master/pkg/security/controller/namespace_scc_allocation_controller.go#L112
+			if k == securityv1.UIDRangeAnnotation {
+				return true, nil
+			}
+		}
+		e2e.Logf("project %s current annotation set: %#v", name, proj.Annotations)
+		return false, nil
+	}
+	return wait.Poll(time.Duration(15*time.Second), 2*time.Minute, waitFn)
+}
+
 // WaitForAnImageStream waits for an ImageStream to fulfill the isOK function
 func WaitForAnImageStream(client imagev1typedclient.ImageStreamInterface,
 	name string,
